package jumpstart.util;


import org.slf4j.Logger;

public class EJBProviderUtil {

	static public EJBProviderEnum detectEJBProvider(Logger logger) {
		logger.info("Looking for an EJB provider...");
		
		try {
			Class.forName("org.apache.openejb.tomcat.loader.TomcatEmbedder");
			logger.info("...found TomCat OpenEJB local.");
			return EJBProviderEnum.TOMCAT_OPENEJB_LOCAL;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("org.apache.openejb.loader.Loader");
			logger.info("...found OpenEJB local.");
			return EJBProviderEnum.OPENEJB_LOCAL;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("org.apache.openejb.client.Client");
			logger.info("...found OpenEJB remote.");
			return EJBProviderEnum.OPENEJB_REMOTE;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("org.jboss.Main");
			logger.info("...found JBoss local.");
			return EJBProviderEnum.JBOSS_LOCAL;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("org.jboss.remoting.Client");
			logger.info("...found JBoss remote.");
			return EJBProviderEnum.JBOSS_REMOTE;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("com.sun.enterprise.ee.tools.launcher.ProcessLauncher");
			logger.info("...found GlassFish (SunAS9) local.");
			return EJBProviderEnum.GLASSFISH_2_LOCAL;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("PLBootstrap");
			logger.info("...found GlassFish (SunAS9) remote.");
			return EJBProviderEnum.GLASSFISH_2_REMOTE;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("com.sun.enterprise.admin.cli.AsadminMain");
			logger.info("...found GlassFish 3 local.");
			return EJBProviderEnum.GLASSFISH_3_LOCAL;
		}
		catch (Exception e) {
		}

		try {
			Class.forName("org.glassfish.appclient.client.acc.agent.ACCAgentClassLoader");
			logger.info("...found GlassFish 3 remote.");
			return EJBProviderEnum.GLASSFISH_3_REMOTE;
		}
		catch (Exception e) {
		}

		throw new IllegalStateException(
				"Failed to detect a known EJBProvider. Tried OpenEJB, JBoss, and GlassFish.");
	}
}
