// Based on an example kindly provided by George Christman and Lance Java.

package jumpstart.web.pages.examples.ajax;

import java.util.List;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Category;
import jumpstart.business.domain.examples.iface.CategoryNode;
import jumpstart.business.domain.examples.iface.ICategoryServiceLocal;
import jumpstart.web.model.examples.tree.CategoryTreeModelAdapter;

import org.apache.tapestry5.ValueEncoder;
import org.apache.tapestry5.annotations.InjectComponent;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Tree;
import org.apache.tapestry5.corelib.components.Zone;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;
import org.apache.tapestry5.services.ajax.AjaxResponseRenderer;
import org.apache.tapestry5.tree.DefaultTreeModel;
import org.apache.tapestry5.tree.TreeModel;
import org.apache.tapestry5.tree.TreeNode;

public class TreeFromDatabaseWithZones {

	// Screen fields

	private TreeModel<CategoryNode> treeModel;

	@Property
	@SuppressWarnings("unused")
	private TreeNode<CategoryNode> treeNode;

	@Property
	private CategoryNode categoryNode;

	@Property
	private Category selectedCategory;

	// Generally useful bits and pieces

	@InjectComponent
	private Tree tree;

	@InjectComponent
	private Zone treeZone;

	@InjectComponent
	private Zone selectedZone;

	@Inject
	private AjaxResponseRenderer ajaxResponseRenderer;
	
	@Inject
	private Request request;

	@EJB
	private ICategoryServiceLocal categoryService;

	// The code

	void onClearExpansions() {
		tree.clearExpansions();

		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(treeZone).addRender(selectedZone);
		}
	}

	void onRefresh() {
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(treeZone).addRender(selectedZone);
		}
	}

	void onLeafSelected(Integer categoryId) {
		CategoryNode categoryNode = categoryService.findCategoryInfo(categoryId);
		selectedCategory = categoryNode.getCategory();
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(treeZone).addRender(selectedZone);
		}
	}

	// Getters and setters

	public TreeModel<CategoryNode> getTreeModel() {
		if (treeModel == null) {
			ValueEncoder<CategoryNode> encoder = new ValueEncoder<CategoryNode>() {

				@Override
				public String toClient(CategoryNode node) {
					return node.getCategory().getId().toString();
				}

				@Override
				public CategoryNode toValue(String node) {
					return categoryService.findCategoryInfo(new Integer(node));
				}
			};

			treeModel = new DefaultTreeModel<CategoryNode>(encoder, new CategoryTreeModelAdapter(categoryService),
					categoryService.findRoots());
		}
		return treeModel;
	}

	public List<CategoryNode> getHasResults() {
		return categoryService.findRoots();
	}

	public String getLeafClass() {
		if (selectedCategory != null && categoryNode.getCategory().equals(selectedCategory)) {
			return "selected";
		}
		else {
			return "";
		}
	}

}
