package jumpstart.web.pages.examples.jodatime;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.DateStuff;
import jumpstart.business.domain.examples.iface.IDateStuffServiceLocal;
import jumpstart.util.ExceptionUtil;

import org.apache.tapestry5.PersistenceConstants;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;

public class JodaTime2 {

	// The activation context

	private Long dateStuffId;

	// Screen fields

	@Property
	private DateStuff dateStuff;

	// Work fields

	// This carries version through the redirect that follows a server-side validation failure.
	@Persist(PersistenceConstants.FLASH)
	private Integer versionFlash;

	// Generally useful bits and pieces

	@EJB
	private IDateStuffServiceLocal dateStuffService;

	@Component(id = "dates")
	private Form form;

	// The code

	Long onPassivate() {
		return dateStuffId;
	}

	void onActivate(Long dateStuffId) {
		this.dateStuffId = dateStuffId;
	}

	// Form bubbles up the PREPARE_FOR_RENDER event during form render.

	void onPrepareForRender() throws Exception {
		dateStuff = findDateStuff(dateStuffId);

		// If the form has errors then we're redisplaying after a redirect.
		// Form will restore your input values but it's up to us to restore Hidden values.

		if (form.getHasErrors()) {
			dateStuff.setVersion(versionFlash);
		}
	}

	// Form bubbles up the PREPARE_FOR_SUBMIT event during form submission.

	void onPrepareForSubmit() throws Exception {
		dateStuff = findDateStuff(dateStuffId);
	}

	private DateStuff findDateStuff(Long dateStuffId) throws Exception {
		// Ask business service to find DateStuff
		DateStuff dateStuff = dateStuffService.findDateStuff(dateStuffId);

		if (dateStuff == null) {
			throw new IllegalStateException("Database data has not been set up!");
		}

		return dateStuff;
	}

	void onValidateFromDates() {
		try {
			dateStuffService.changeDateStuff(dateStuff);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			form.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	Object onSuccess() {
		return JodaTime1.class;
	}
	
	void onFailure() {
		versionFlash = dateStuff.getVersion();
	}

	void onRefresh() {
		// By doing nothing the page will be displayed afresh.
	}

	public String getDateFieldFormat() {
		return "dd/MM/yyyy";
	}
}
