package jumpstart.web.pages.together.ajaxfiltercrud;

import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.List;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.Regions;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;
import jumpstart.web.components.CustomForm;

import org.apache.tapestry5.PersistenceConstants;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectComponent;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Grid;
import org.apache.tapestry5.corelib.components.Zone;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;
import org.apache.tapestry5.services.ajax.AjaxResponseRenderer;

public class Persons {

	private final String demoModeStr = System.getProperty("jumpstart.demo-mode");
	static private final int MAX_RESULTS = 40;

	private enum Mode {
		CREATE, REVIEW, UPDATE;
	}

	// Screen fields

	@Property
	@Persist
	private boolean highlightZoneUpdates;

	@Property
	@Persist
	private String partialName;

	@Property
	@Persist
	@SuppressWarnings("unused")
	private List<Person> listPersons;

	@Property
	private Person listPerson;

	@Property
	private Long highlightPersonId;

	@Property
	private Mode mode;

	@Property
	private Long personId;

	@Property
	private Person person;

	@Property
	@Persist(PersistenceConstants.FLASH)
	@SuppressWarnings("unused")
	private String deleteMessage;

	@Property
	@Component(id = "list")
	@SuppressWarnings("unused")
	private Grid list;

	// Generally useful bits and pieces

	@EJB
	private IPersonServiceLocal personService;

	@Component(id = "createForm")
	private CustomForm createForm;

	@Component(id = "updateForm")
	private CustomForm updateForm;

	@InjectComponent
	private Zone listZone;

	@InjectComponent
	private Zone editorZone;

	@Inject
	private Messages messages;

	@Inject
	private Request request;

	@Inject
	private AjaxResponseRenderer ajaxResponseRenderer;

	// The code

	// setupRender() is called by Tapestry right before it starts rendering the page.

	void setupRender() {
		refreshList();
	}

	void refreshList() {
		listPersons = personService.findPersons(partialName, MAX_RESULTS);
		highlightPersonId = personId;
		if (highlightPersonId == null) {
			// Null is a nuisance because it fails to match the signature of our onSuccess method.
			highlightPersonId = -1L;
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// FILTER
	// /////////////////////////////////////////////////////////////////////

	void onSuccessFromFilterForm(Long highlightPersonId) {
		personId = highlightPersonId;
		
		if (request.isXHR()) {
			refreshList();
			ajaxResponseRenderer.addRender(listZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toCreate"

	void onToCreate() {
		mode = Mode.CREATE;
		personId = null;

		if (request.isXHR()) {
			refreshList();
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// Handle event "cancelCreate"

	void onCancelCreate() {
		mode = null;
		personId = null;

		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Component "createForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromCreateForm() throws Exception {
		mode = Mode.CREATE;
		// Instantiate a Person for the form data to overlay.
		person = new Person();
	}

	// Component "createForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromCreateForm() {

		if (createForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		if (demoModeStr != null && demoModeStr.equals("true")) {
			createForm.recordError("Sorry, but Create is not allowed in Demo mode.");
			return;
		}

		try {
			person = personService.createPerson(person);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			createForm.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	// Component "createForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether VALIDATE
	// records an error

	void onSuccessFromCreateForm() {
		mode = Mode.REVIEW;
		personId = person.getId();

		if (request.isXHR()) {
			refreshList();
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	void onFailureFromCreateForm() {
		mode = Mode.CREATE;
		personId = null;

		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// Handle event "selected"

	void onSelected(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;

		if (request.isXHR()) {
			refreshList();
			person = personService.findPerson(personId);
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate"

	void onToUpdate(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Handle event "cancelUpdate"

	void onCancelUpdate(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;

		if (request.isXHR()) {
			person = personService.findPerson(personId);
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Component "updateForm" bubbles up the PREPARE_FOR_RENDER event when it is rendered

	void onPrepareForRenderFromUpdateForm(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		
		// If the form is valid then we're not redisplaying due to error, so get the person.

		if (updateForm.isValid()) {
			person = personService.findPerson(personId);
			// Handle null person in the template.
		}
	}

	// Component "updateForm" bubbles up the PREPARE_FOR_SUBMIT event when it is submitted

	void onPrepareForSubmitFromUpdateForm(Long personId) {
		this.personId = personId;
		
		// Get objects for the form fields to overlay.
		person = personService.findPerson(personId);
		
		if (person == null) {
			person = new Person();
			updateForm.recordError("Person has been deleted by another process.");
		}
	}

	// Component "updateForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromUpdateForm() {

		if (updateForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}
		
		try {
			personService.changePerson(person);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			updateForm.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	// Component "updateForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether VALIDATE
	// records an error

	void onSuccessFromUpdateForm() {
		mode = Mode.REVIEW;

		if (request.isXHR()) {
			refreshList();
			person = personService.findPerson(personId);
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	void onFailureFromUpdateForm() {
		mode = Mode.UPDATE;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "delete"

	void onDelete(Long personId, Integer personVersion) {

		this.personId = personId;
		boolean successfulDelete = false;

		if (demoModeStr != null && demoModeStr.equals("true")) {
			deleteMessage = "Sorry, but Delete is not allowed in Demo mode.";
		}
		else {

			try {
				personService.deletePerson(personId, personVersion);
				successfulDelete = true;
			}
			catch (Exception e) {
				// Display the cause. In a real system we would try harder to get a user-friendly message.
				deleteMessage = ExceptionUtil.getRootCauseMessage(e);
			}
			
		}

		if (successfulDelete) {
			mode = null;
			this.personId = null;

			if (request.isXHR()) {
				refreshList();
				ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
			}
		}
		else {
			mode = Mode.REVIEW;
			this.personId = personId;

			if (request.isXHR()) {
				person = personService.findPerson(personId);
				ajaxResponseRenderer.addRender(editorZone);
			}
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// OTHER
	// /////////////////////////////////////////////////////////////////////

	public String getLinkCSSClass() {
		if (listPerson != null && listPerson.getId().equals(highlightPersonId)) {
			return "active";
		}
		else {
			return "";
		}
	}

	public String getZoneUpdateFunction() {
		return highlightZoneUpdates ? "highlight" : "show";
	}

	public boolean isModeCreate() {
		return mode == Mode.CREATE;
	}

	public boolean isModeReview() {
		return mode == Mode.REVIEW;
	}

	public boolean isModeUpdate() {
		return mode == Mode.UPDATE;
	}

	public boolean isModeNull() {
		return mode == null;
	}

	public String getPersonRegion() {
		// Follow the same naming convention that the Select component uses
		return messages.get(Regions.class.getSimpleName() + "." + person.getRegion().name());
	}

	public String getDatePattern() {
		return "dd/MM/yyyy";
	}

	public Format getDateFormat() {
		return new SimpleDateFormat(getDatePattern());
	}
}
