package jumpstart.web.pages.together.withlayout.componentscrud;

import jumpstart.web.components.together.componentscrud.PersonEditor.Mode;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.annotations.ActivationRequestParameter;
import org.apache.tapestry5.annotations.Property;

public class Persons {

	// The activation context

	@Property
	private Mode mode;

	@Property
	private Long personId;

	@Property
	@SuppressWarnings("unused")
	private Long highlightPersonId;

	// Query string parameters

	@ActivationRequestParameter(value = "filter")
	@Property
	@SuppressWarnings("unused")
	private String partialName;

	// The code

	// onPassivate() is called by Tapestry to get the activation context to put in the URL.

	Object[] onPassivate() {

		if (mode == null) {
			return null;
		}
		else if (mode == Mode.CREATE) {
			return new Object[] { mode };
		}
		else if (mode == Mode.REVIEW || mode == Mode.UPDATE) {
			return new Object[] { mode, personId };
		}
		else {
			throw new IllegalStateException(mode.toString());
		}

	}

	// onActivate() is called by Tapestry to pass in the activation context from the URL.

	void onActivate(EventContext ec) {

		if (ec.getCount() == 0) {
			mode = null;
			personId = null;
		}
		else if (ec.getCount() == 1) {
			mode = ec.get(Mode.class, 0);
			personId = null;
		}
		else {
			mode = ec.get(Mode.class, 0);
			personId = ec.get(Long.class, 1);
		}

	}

	// setupRender() is called by Tapestry right before it starts rendering the page.

	void setupRender() {
		highlightPersonId = personId;
	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toCreate" from this page

	void onToCreate() {
		mode = Mode.CREATE;
		personId = null;
	}

	// Handle event "cancelCreate" from component "editor"

	void onCancelCreateFromEditor() {
		mode = null;
		personId = null;
	}

	// Handle event "successfulCreate" from component "editor"

	void onSuccessfulCreateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
	}

	// Handle event "failedCreate" from component "editor"

	void onFailedCreateFromEditor() {
		mode = Mode.CREATE;
		this.personId = null;
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// Handle event "selected" from component "list"

	void onSelectedFromList(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
	}

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate" from component "editor"

	void onToUpdateFromEditor(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
	}

	// Handle event "cancelUpdate" from component "editor"

	void onCancelUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
	}

	// Handle event "successfulUpdate" from component "editor"

	void onSuccessfulUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
	}

	// Handle event "failedUpdate" from component "editor"

	void onFailedUpdateFromEditor(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "successfulDelete" from component "editor"

	void onSuccessfulDeleteFromEditor(Long personId) {
		mode = null;
		this.personId = null;
	}

	// Handle event "failedDelete" from component "editor"

	void onFailedDeleteFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
	}

}
