package jumpstart.web.pages.together.withlayout2.gracefulajaxcomponentscrud;

import jumpstart.web.components.together.gracefulajaxcomponentscrud.PersonEditor.Mode;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.annotations.InjectComponent;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Zone;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;
import org.apache.tapestry5.services.ajax.AjaxResponseRenderer;

public class Persons {

	// The activation context

	@Property
	private Mode mode;

	@Property
	private Long personId;

	// Screen fields

	@Property
	@Persist
	private boolean highlightZoneUpdates;

	@Property
	@SuppressWarnings("unused")
	private Long highlightPersonId;

	// Generally useful bits and pieces

	@InjectComponent
	private Zone listZone;

	@InjectComponent
	private Zone editorZone;

	@Inject
	private Request request;

	@Inject
	private AjaxResponseRenderer ajaxResponseRenderer;

	// The code

	// onPassivate() is called by Tapestry to get the activation context to put in the URL.

	Object[] onPassivate() {

		if (mode == null) {
			return null;
		}
		else if (mode == Mode.CREATE) {
			return new Object[] { mode };
		}
		else if (mode == Mode.REVIEW || mode == Mode.UPDATE || mode == Mode.CONFIRM_DELETE) {
			return new Object[] { mode, personId };
		}
		else {
			throw new IllegalStateException(mode.toString());
		}

	}

	// onActivate() is called by Tapestry to pass in the activation context from the URL.

	void onActivate(EventContext ec) {

		if (ec.getCount() == 0) {
			mode = null;
			personId = null;
		}
		else if (ec.getCount() == 1) {
			mode = ec.get(Mode.class, 0);
			personId = null;
		}
		else {
			mode = ec.get(Mode.class, 0);
			personId = ec.get(Long.class, 1);
		}

	}

	// setupRender() is called by Tapestry right before it starts rendering the page.

	void setupRender() {
		highlightPersonId = personId;
	}

	// /////////////////////////////////////////////////////////////////////
	// FILTER
	// /////////////////////////////////////////////////////////////////////

	// Handle event "filter" from component "list"

	void onFilterFromList(Long highlightPersonId) {
		this.highlightPersonId = highlightPersonId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(listZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toCreate" from this page

	void onToCreate() {
		mode = Mode.CREATE;
		personId = null;
		highlightPersonId = null;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// Handle event "cancelCreate" from component "editor"

	void onCancelCreateFromEditor() {
		mode = null;
		personId = null;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Handle event "successfulCreate" from component "editor"

	void onSuccessfulCreateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// Handle event "failedCreate" from component "editor"

	void onFailedCreateFromEditor() {
		mode = Mode.CREATE;
		personId = null;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// Handle event "selected" from component "list"

	void onSelectedFromList(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate" from component "editor"

	void onToUpdateFromEditor(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Handle event "cancelUpdate" from component "editor"

	void onCancelUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Handle event "successfulUpdate" from component "editor"

	void onSuccessfulUpdateFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		highlightPersonId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// Handle event "failedUpdate" from component "editor"

	void onFailedUpdateFromEditor(Long personId) {
		mode = Mode.UPDATE;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "successfulDelete" from component "editor"

	void onSuccessfulDeleteFromEditor(Long personId) {
		mode = null;
		this.personId = null;
		highlightPersonId = null;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(listZone).addRender(editorZone);
		}
	}

	// Handle event "failedDelete" from component "editor"

	void onFailedDeleteFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// CONFIRM DELETE - used only when JavaScript is disabled.
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toConfirmDelete" from component "editor"

	void onToConfirmDeleteFromEditor(Long personId) {
		mode = Mode.CONFIRM_DELETE;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Handle event "cancelConfirmDelete" from component "editor"

	Object onCancelConfirmDeleteFromEditor(Long personId) {
		mode = Mode.REVIEW;
		this.personId = null;
		highlightPersonId = personId;
		return null;
	}

	// Handle event "successfulConfirmDelete" from component "editor"

	void onSuccessfulConfirmDeleteFromEditor(Long personId) {
		mode = null;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// Handle event "failedConfirmDelete" from component "editor"

	void onFailedConfirmDeleteFromEditor(Long personId) {
		mode = Mode.CONFIRM_DELETE;
		this.personId = personId;
		
		if (request.isXHR()) {
			ajaxResponseRenderer.addRender(editorZone);
		}
	}

	// /////////////////////////////////////////////////////////////////////
	// GETTERS
	// /////////////////////////////////////////////////////////////////////

	public String getZoneUpdateFunction() {
		return highlightZoneUpdates ? "highlight" : "show";
	}

}
