package jumpstart.web.services;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLConnection;

import org.apache.tapestry5.ioc.services.ClasspathURLConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ClasspathURLConverterJBoss5 implements ClasspathURLConverter {
	private static Logger logger = LoggerFactory.getLogger(ClasspathURLConverterJBoss5.class);

	public URL convert(URL url) {

		// If the URL is a virtual URL (JBoss 5 uses a Virtual File System)...
		
		if (url != null && url.getProtocol().startsWith("vfs")) {

			// Ask the VFS what the physical URL is...
			
			try {
				URLConnection connection = url.openConnection();
				Object jBossVirtualFile = connection.getContent();
				// Use reflection so that we don't need JBoss in the classpath at compile time.
				Object zipEntryHandler = invoke(jBossVirtualFile, "getHandler");
				Object realUrl = invoke(zipEntryHandler, "getRealURL");
				return (URL) realUrl;
			}
			catch (Exception e) {
				if (e.getCause() == null) {
					logger.error(e.toString());
				}
				else {
					logger.error(e.getCause().toString());
				}
			}
		}
		return url;
	}

	private Object invoke(Object target, String methodName) throws NoSuchMethodException, InvocationTargetException,
			IllegalAccessException {
		Class<?> type = target.getClass();
		Method method;
		try {
			method = type.getMethod(methodName);
		}
		catch (NoSuchMethodException e) {
			method = type.getDeclaredMethod(methodName);
			method.setAccessible(true);
		}
		return method.invoke(target);
	}

}
